@@ -53,11 +53,12 @@ class EmitCModel final : public EmitCFunc {
     }
 
     void emitHeader(AstNodeModule* modp) {
-        UASSERT(!m_ofp, "Output file should not be open");
+        UASSERT(!ofp(), "Output file should not be open");
 
         const string filename = v3Global.opt.makeDir() + "/" + topClassName() + ".h";
-        newCFile(filename, /* slow: */ false, /* source: */ false);
-        m_ofp = v3Global.opt.systemC() ? new V3OutScFile{filename} : new V3OutCFile{filename};
+        setOutputFile(v3Global.opt.systemC() ? new V3OutScFile{filename}
+                                             : new V3OutCFile{filename},
+                      newCFile(filename, /* slow: */ false, /* source: */ false));
 
         ofp()->putsHeader();
         puts("// DESCRIPTION: Verilator output: Primary model header\n");
@@ -118,6 +119,17 @@ class EmitCModel final : public EmitCFunc {
                 }
             }
         }
+        if (v3Global.opt.emitAccessors()) {
+            puts("\n// ACCESSORS\n"
+                 "// The application code should use these methods to\n"
+                 "// propagate new values into/out from the Verilated model\n"
+                 "// instead of using signal variables directly.\n");
+            for (const AstNode* nodep = modp->stmtsp(); nodep; nodep = nodep->nextp()) {
+                if (const AstVar* const varp = VN_CAST(nodep, Var)) {
+                    if (varp->isPrimaryIO()) emitVarAccessors(varp);
+                }
+            }
+        }
         if (optSystemC() && v3Global.usesTiming()) puts("sc_core::sc_event trigger_eval;\n");
 
         // Cells instantiated by the top level (for access to /* verilator public */)
@@ -258,7 +270,7 @@ class EmitCModel final : public EmitCFunc {
 
         ofp()->putsEndGuard();
 
-        VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
+        closeOutputFile();
     }
 
     void emitConstructorImplementation(AstNodeModule* modp) {
@@ -601,11 +613,12 @@ class EmitCModel final : public EmitCFunc {
     }
 
     void emitImplementation(AstNodeModule* modp) {
-        UASSERT(!m_ofp, "Output file should not be open");
+        UASSERT(!ofp(), "Output file should not be open");
 
         const string filename = v3Global.opt.makeDir() + "/" + topClassName() + ".cpp";
-        newCFile(filename, /* slow: */ false, /* source: */ true);
-        m_ofp = v3Global.opt.systemC() ? new V3OutScFile{filename} : new V3OutCFile{filename};
+        setOutputFile(v3Global.opt.systemC() ? new V3OutScFile{filename}
+                                             : new V3OutCFile{filename},
+                      newCFile(filename, /* slow: */ false, /* source: */ true));
 
         ofp()->putsHeader();
         puts("// DESCRIPTION: Verilator output: "
@@ -624,11 +637,11 @@ class EmitCModel final : public EmitCFunc {
         if (v3Global.opt.trace()) emitTraceMethods(modp);
         if (v3Global.opt.savable()) emitSerializationFunctions();
 
-        VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
+        closeOutputFile();
     }
 
     void emitDpiExportDispatchers(AstNodeModule* modp) {
-        UASSERT(!m_ofp, "Output file should not be open");
+        UASSERT(!ofp(), "Output file should not be open");
 
         // Emit DPI Export dispatchers
         for (AstNode* nodep = modp->stmtsp(); nodep; nodep = nodep->nextp()) {
@@ -639,19 +652,19 @@ class EmitCModel final : public EmitCFunc {
                 // Splitting file, so using parallel build.
                 v3Global.useParallelBuild(true);
                 // Close old file
-                VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
+                closeOutputFile();
             }
 
-            if (!m_ofp) {
+            if (!ofp()) {
                 string filename = v3Global.opt.makeDir() + "/" + topClassName() + "__Dpi_Export";
                 filename = m_uniqueNames.get(filename);
                 filename += ".cpp";
-                newCFile(filename, /* slow: */ false, /* source: */ true);
-                m_ofp = v3Global.opt.systemC() ? new V3OutScFile{filename}
-                                               : new V3OutCFile{filename};
+                setOutputFile(v3Global.opt.systemC() ? new V3OutScFile{filename}
+                                                     : new V3OutCFile{filename},
+                              newCFile(filename, /* slow: */ false, /* source: */ true));
                 splitSizeReset();  // Reset file size tracking
                 m_lazyDecls.reset();
-                m_ofp->putsHeader();
+                ofp()->putsHeader();
                 puts(
                     "// DESCRIPTION: Verilator output: Implementation of DPI export functions.\n");
                 puts("//\n");
@@ -664,7 +677,7 @@ class EmitCModel final : public EmitCFunc {
             iterateConst(funcp);
         }
 
-        if (m_ofp) VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
+        if (ofp()) closeOutputFile();
     }
 
     void main(AstNodeModule* modp) {
