@@ -375,7 +375,10 @@ class EmitVBaseVisitorConst VL_NOT_FINAL : public EmitCBaseVisitorConst {
         iterateAndNextConstNull(nodep->lhsp());
         puts(";\n");
     }
-    void visit(AstStop* nodep) override { putfs(nodep, "$stop;\n"); }
+    void visit(AstStop* nodep) override {
+        emitVerilogFormat(nodep, nodep->emitVerilog());
+        puts(";\n");
+    }
     void visit(AstFinish* nodep) override { putfs(nodep, "$finish;\n"); }
     void visit(AstStmtExpr* nodep) override {
         iterateConst(nodep->exprp());
@@ -602,7 +605,7 @@ class EmitVBaseVisitorConst VL_NOT_FINAL : public EmitCBaseVisitorConst {
     }
     void visit(AstTypedef* nodep) override {
         putfs(nodep, "typedef ");
-        iterateAndNextConstNull(nodep->dtypep());
+        iterateAndNextConstNull(nodep->subDTypep());
         puts(" ");
         puts(nodep->prettyName());
         puts(";\n");
@@ -630,7 +633,13 @@ class EmitVBaseVisitorConst VL_NOT_FINAL : public EmitCBaseVisitorConst {
         iterateConst(nodep->subDTypep());
         iterateAndNextConstNull(nodep->rangep());
     }
-    void visit(AstRefDType* nodep) override { iterateConst(nodep->skipRefp()); }
+    void visit(AstRefDType* nodep) override {
+        if (nodep->subDTypep()) {
+            iterateConst(nodep->skipRefp());
+        } else {
+            puts("\n???? // "s + nodep->prettyTypeName() + " -> UNLINKED\n");
+        }
+    }
     void visit(AstNodeUOrStructDType* nodep) override {
         puts(nodep->verilogKwd() + " ");
         if (nodep->packed()) puts("packed ");
