@@ -21,6 +21,7 @@
 //              (Interfaces also matter, as if a module is parameterized
 //              this effectively changes the width behavior of all that
 //              reference the iface.)
+//
 //              Clone module cell calls, renaming with __{par1}_{par2}_...
 //              Substitute constants for cell's module's parameters.
 //              Relink pins and cell and ifacerefdtype to point to new module.
@@ -50,6 +51,7 @@
 
 #include "V3Case.h"
 #include "V3Const.h"
+#include "V3EmitV.h"
 #include "V3Hasher.h"
 #include "V3Os.h"
 #include "V3Parse.h"
@@ -64,7 +66,7 @@
 VL_DEFINE_DEBUG_FUNCTIONS;
 
 //######################################################################
-// Hierarchical block and parameter db (modules without parameter is also handled)
+// Hierarchical block and parameter db (modules without parameters are also handled)
 
 class ParameterizedHierBlocks final {
     using HierBlockOptsByOrigName = std::multimap<std::string, const V3HierarchicalBlockOption*>;
@@ -90,7 +92,9 @@ class ParameterizedHierBlocks final {
 
 public:
     ParameterizedHierBlocks(const V3HierBlockOptSet& hierOpts, AstNetlist* nodep)
-        : m_hierSubRun(!v3Global.opt.hierBlocks().empty() || v3Global.opt.hierChild()) {
+        : m_hierSubRun((!v3Global.opt.hierBlocks().empty() || v3Global.opt.hierChild())
+                       // Exclude consolidation
+                       && !v3Global.opt.hierParamFile().empty()) {
         for (const auto& hierOpt : hierOpts) {
             m_hierBlockOptsByOrigName.emplace(hierOpt.second.origName(), &hierOpt.second);
             const V3HierarchicalBlockOption::ParamStrMap& params = hierOpt.second.params();
@@ -146,8 +150,6 @@ public:
             UASSERT(params.size() == hierIt->second->params().size(), "not match");
             for (AstPin* pinp = firstPinp; pinp; pinp = VN_AS(pinp->nextp(), Pin)) {
                 if (!pinp->exprp()) continue;
-                UASSERT_OBJ(!pinp->modPTypep(), pinp,
-                            "module with type parameter must not be a hierarchical block");
                 if (const AstVar* const modvarp = pinp->modVarp()) {
                     AstConst* const constp = VN_AS(pinp->exprp(), Const);
                     UASSERT_OBJ(constp, pinp,
@@ -199,7 +201,7 @@ public:
                 varNum.opIToRD(pinValuep->num());
                 var = varNum.toDouble();
             }
-            return v3EpsilonEqual(var, hierOptParamp->num().toDouble());
+            return V3Number::epsilonEqual(var, hierOptParamp->num().toDouble());
         } else {  // Now integer type is assumed
             // Bitwidth of hierOptParamp is accurate because V3Width already calculated in the
             // previous run. Bitwidth of pinValuep is before width analysis, so pinValuep is casted
@@ -230,10 +232,10 @@ class ParamProcessor final {
     //                          //        (0=not processed, 1=iterated, but no number,
     //                          //         65+ parameter numbered)
     // NODE STATE - Shared with ParamVisitor
-    //   AstClass::user3p()     // AstClass* Unchanged copy of the parameterized class node.
-    //                                    The class node may be modified according to parameter
-    //                                    values and an unchanged copy is needed to instantiate
-    //                                    classes with different parameters.
+    //   AstNodeModule::user3p()  // AstNodeModule* Unaltered copy of the parameterized module.
+    //                               The module/class node may be modified according to parameter
+    //                               values and an unchanged copy is needed to instantiate
+    //                               modules/classes with different parameters.
     //   AstNodeModule::user2() // bool   True if processed
     //   AstGenFor::user2()     // bool   True if processed
     //   AstVar::user2()        // bool   True if constant propagated
@@ -275,7 +277,7 @@ class ParamProcessor final {
     // Database to get lib-create wrapper that matches parameters in hierarchical Verilation
     ParameterizedHierBlocks m_hierBlocks;
     // Default parameter values key:parameter name, value:default value (can be nullptr)
-    using DefaultValueMap = std::map<std::string, AstConst*>;
+    using DefaultValueMap = std::map<std::string, AstNode*>;
     // Default parameter values of hierarchical blocks
     std::map<AstNodeModule*, DefaultValueMap> m_defaultParameterValues;
     VNDeleter m_deleter;  // Used to delay deletion of nodes
@@ -346,7 +348,24 @@ class ParamProcessor final {
             if (dtypep->isRanged()) {
                 key += "[" + cvtToStr(dtypep->left()) + ":" + cvtToStr(dtypep->right()) + "]";
             }
+        } else if (const AstPackArrayDType* const dtypep = VN_CAST(nodep, PackArrayDType)) {
+            key += "[";
+            key += cvtToStr(dtypep->left());
+            key += ":";
+            key += cvtToStr(dtypep->right());
+            key += "] ";
+            key += paramValueString(dtypep->subDTypep());
+        } else if (const AstInitArray* const initp = VN_CAST(nodep, InitArray)) {
+            key += "{";
+            for (auto it : initp->map()) {
+                key += paramValueString(it.second->valuep());
+                key += ",";
+            }
+            key += "}";
+        } else if (const AstNodeDType* const dtypep = VN_CAST(nodep, NodeDType)) {
+            key += dtypep->prettyDTypeName(true);
         }
+        UASSERT_OBJ(!key.empty(), nodep, "Parameter yielded no value string");
         return key;
     }
 
@@ -457,7 +476,7 @@ class ParamProcessor final {
     }
     // Check if parameter setting during instantiation is simple enough for hierarchical Verilation
     void checkSupportedParam(AstNodeModule* modp, AstPin* pinp) const {
-        // InitArray and AstParamTypeDType are not supported because that can not be set via -G
+        // InitArray is not supported because that can not be set via -G
         // option.
         if (pinp->modVarp()) {
             bool supported = false;
@@ -465,13 +484,11 @@ class ParamProcessor final {
                 supported = !constp->isOpaque();
             }
             if (!supported) {
-                pinp->v3error(AstNode::prettyNameQ(modp->origName())
-                              << " has hier_block metacomment, hierarchical Verilation"
-                              << " supports only integer/floating point/string parameters");
+                pinp->v3error(
+                    AstNode::prettyNameQ(modp->origName())
+                    << " has hier_block metacomment, hierarchical Verilation"
+                    << " supports only integer/floating point/string and type param parameters");
             }
-        } else {
-            pinp->v3error(AstNode::prettyNameQ(modp->origName())
-                          << " has hier_block metacomment, but 'parameter type' is not supported");
         }
     }
     bool moduleExists(const string& modName) const {
@@ -487,16 +504,18 @@ class ParamProcessor final {
         //  - Hash the long name to get valid Verilog symbol
         UASSERT_OBJ(modp->hierBlock(), modp, "should be used for hierarchical block");
 
-        std::map<string, AstConst*> pins;
-        for (AstPin* pinp = paramPinsp; pinp; pinp = VN_AS(pinp->nextp(), Pin)) {
+        std::map<string, AstNode*> pins;
+
+        AstPin* pinp = paramPinsp;
+        while (pinp) {
             checkSupportedParam(modp, pinp);
             if (const AstVar* const varp = pinp->modVarp()) {
                 if (!pinp->exprp()) continue;
-                if (varp->isGParam()) {
-                    AstConst* const constp = VN_CAST(pinp->exprp(), Const);
-                    pins.emplace(varp->name(), constp);
-                }
+                if (varp->isGParam()) { pins.emplace(varp->name(), pinp->exprp()); }
+            } else if (VN_IS(pinp->exprp(), BasicDType) || VN_IS(pinp->exprp(), NodeDType)) {
+                pins.emplace(pinp->name(), pinp->exprp());
             }
+            pinp = VN_AS(pinp->nextp(), Pin);
         }
 
         const auto pair = m_defaultParameterValues.emplace(
@@ -513,6 +532,8 @@ class ParamProcessor final {
                         // nullptr means that the parameter is using some default value.
                         params.emplace(varp->name(), constp);
                     }
+                } else if (const AstParamTypeDType* const p = VN_CAST(stmtp, ParamTypeDType)) {
+                    params.emplace(p->name(), p->skipRefp());
                 }
             }
             pair.first->second = std::move(params);
@@ -523,13 +544,23 @@ class ParamProcessor final {
         string longname = modp->origName();
         for (auto&& defaultValue : paramsIt->second) {
             const auto pinIt = pins.find(defaultValue.first);
-            const AstConst* const constp
-                = pinIt == pins.end() ? defaultValue.second : pinIt->second;
+            // If the pin does not have a value assigned, use the default one.
+            const AstNode* const node = pinIt == pins.end() ? defaultValue.second : pinIt->second;
             // This longname is not valid as verilog symbol, but ok, because it will be hashed
             longname += "_" + defaultValue.first + "=";
             // constp can be nullptr
-            if (constp) longname += constp->num().ascii(false);
+
+            if (const AstConst* const p = VN_CAST(node, Const)) {
+                // Treat modules parametrized with the same values but with different type as the
+                // same.
+                longname += p->num().ascii(false);
+            } else if (node) {
+                std::stringstream type;
+                V3EmitV::verilogForTree(node, type);
+                longname += type.str();
+            }
         }
+        UINFO(9, "       module params longname: " << longname << endl);
 
         const auto iter = m_longMap.find(longname);
         if (iter != m_longMap.end()) return iter->second;  // Already calculated
@@ -737,7 +768,8 @@ class ParamProcessor final {
                 }
             }
         } else if (AstParamTypeDType* const modvarp = pinp->modPTypep()) {
-            AstNodeDType* const exprp = VN_CAST(pinp->exprp(), NodeDType);
+            AstNodeDType* rawTypep = VN_CAST(pinp->exprp(), NodeDType);
+            AstNodeDType* const exprp = rawTypep ? rawTypep->skipRefToEnump() : nullptr;
             const AstNodeDType* const origp = modvarp->skipRefToEnump();
             if (!exprp) {
                 pinp->v3error("Parameter type pin value isn't a type: Param "
@@ -949,7 +981,9 @@ public:
         if (debug() >= 10) nodep->dumpTree("-  cell: ");
         // Evaluate all module constants
         V3Const::constifyParamsEdit(nodep);
-        srcModpr->someInstanceName(someInstanceName + "." + nodep->name());
+        // Set name for warnings for when we param propagate the module
+        const string instanceName = someInstanceName + "." + nodep->name();
+        srcModpr->someInstanceName(instanceName);
 
         if (auto* cellp = VN_CAST(nodep, Cell)) {
             cellDeparam(cellp, srcModpr);
@@ -963,6 +997,9 @@ public:
             nodep->v3fatalSrc("Expected module parameterization");
         }
 
+        // Set name for later warnings (if srcModpr changed value due to cloning)
+        srcModpr->someInstanceName(instanceName);
+
         UINFO(8, "     Done with " << nodep << endl);
         // if (debug() >= 10)
         // v3Global.rootp()->dumpTreeFile(v3Global.debugFilename("param-out.tree"));
@@ -1057,7 +1094,7 @@ class ParamVisitor final : public VNVisitor {
                 UASSERT_OBJ(srcModp, cellp, "Unlinked class ref");
 
                 // Update path
-                string someInstanceName(modp->someInstanceName());
+                string someInstanceName = modp->someInstanceName();
                 if (const string* const genHierNamep = cellp->user2u().to<string*>()) {
                     someInstanceName += *genHierNamep;
                     cellp->user2p(nullptr);
