@@ -995,7 +995,7 @@ int V3PreProcImp::getStateToken() {
         int tok = getRawToken();
 
         // Most states emit white space and comments between tokens. (Unless collecting a string)
-        if (tok == VP_WHITE && state() != ps_STRIFY) return tok;
+        if (tok == VP_WHITE && state() != ps_STRIFY && state() != ps_JOIN) return tok;
         if (tok == VP_BACKQUOTE && state() != ps_STRIFY) tok = VP_TEXT;
         if (tok == VP_COMMENT) {
             if (!m_off) {
@@ -1223,8 +1223,8 @@ int V3PreProcImp::getStateToken() {
                 if (VString::removeWhitespace(string{yyourtext(), yyourleng()}).empty()) {
                     return tok;
                 } else {
-                    error(std::string{"Syntax error in `ifdef () expression; unexpected: '"}
-                          + tokenName(tok) + "'\n");
+                    error("Syntax error in `ifdef () expression; unexpected: '"s + tokenName(tok)
+                          + "'\n");
                 }
                 goto next_tok;
             }
@@ -1254,10 +1254,10 @@ int V3PreProcImp::getStateToken() {
             static string newlines;
             newlines = "\n";  // Always start with trailing return
             if (tok == VP_DEFVALUE) {
-                if (debug() >= 5) {
+                if (debug() >= 5) {  // LCOV_EXCL_START
                     cout << "DefValue='" << V3PreLex::cleanDbgStrg(m_lexp->m_defValue)
                          << "'  formals='" << V3PreLex::cleanDbgStrg(m_formals) << "'\n";
-                }
+                }  // LCOV_EXCL_STOP
                 // Add any formals
                 const string formals = m_formals;
                 string value = m_lexp->m_defValue;
@@ -1326,8 +1326,8 @@ int V3PreProcImp::getStateToken() {
                     // NOP: out = m_preprocp->defSubstitute(out);
                 }
                 VL_DO_DANGLING(m_defRefs.pop(), refp);
-                if (m_defRefs.empty()) {
-                    statePop();
+                statePop();
+                if (m_defRefs.empty() || state() == ps_STRIFY || state() == ps_JOIN) {
                     if (state()
                         == ps_JOIN) {  // Handle {left}```FOO(ARG) where `FOO(ARG) might be empty
                         UASSERT(!m_joinStack.empty(), "`` join stack empty, but in a ``");
@@ -1345,7 +1345,8 @@ int V3PreProcImp::getStateToken() {
                         unputDefrefString("__IF_OFF_IGNORED_DEFINE");
                     }
                     m_lexp->m_parenLevel = 0;
-                } else {  // Finished a defref inside a upper defref
+                } else {  // Finished a defref inside a upper defref,
+                          // and not under stringification or join.
                     // Can't subst now, or
                     // `define a(ign) x,y
                     // foo(`a(ign),`b)  would break because a contains comma
@@ -1353,7 +1354,7 @@ int V3PreProcImp::getStateToken() {
                     refp->nextarg(refp->nextarg() + m_lexp->m_defValue + out);
                     m_lexp->m_defValue = "";
                     m_lexp->m_parenLevel = refp->parenLevel();
-                    statePop();  // Will go to ps_DEFARG, as we're under another define
+                    // Will go to ps_DEFARG, as we're under another define
                 }
                 goto next_tok;
             } else if (tok == VP_DEFREF) {
@@ -1417,7 +1418,7 @@ int V3PreProcImp::getStateToken() {
             }
         }
         case ps_JOIN: {
-            if (tok == VP_SYMBOL || tok == VP_TEXT || tok == VP_STRING) {
+            if (tok == VP_SYMBOL || tok == VP_TEXT || tok == VP_STRING || tok == VP_WHITE) {
                 UASSERT(!m_joinStack.empty(), "`` join stack empty, but in a ``");
                 const string lhs = m_joinStack.top();
                 m_joinStack.pop();
@@ -1429,7 +1430,7 @@ int V3PreProcImp::getStateToken() {
                 unputString(out);
                 statePop();
                 goto next_tok;
-            } else if (tok == VP_EOF || tok == VP_WHITE || tok == VP_COMMENT) {
+            } else if (tok == VP_EOF || tok == VP_COMMENT) {
                 // Other compilers just ignore this, so no warning
                 // "Expecting symbol to terminate ``; whitespace etc cannot
                 // follow ``. Found: "+tokenName(tok)+"\n"
@@ -1551,7 +1552,7 @@ int V3PreProcImp::getStateToken() {
                     }
                     // Similar code in parenthesized define (Search for END_OF_DEFARG)
                     // NOP: out = m_preprocp->defSubstitute(out);
-                    if (m_defRefs.empty()) {
+                    if (m_defRefs.empty() || state() == ps_STRIFY || state() == ps_JOIN) {
                         // Just output the substitution
                         if (state() == ps_JOIN) {  // Handle {left}```FOO where `FOO might be empty
                             UASSERT(!m_joinStack.empty(), "`` join stack empty, but in a ``");
@@ -1569,7 +1570,7 @@ int V3PreProcImp::getStateToken() {
                             unputDefrefString("__IF_OFF_IGNORED_DEFINE");
                         }
                     } else {
-                        // Inside another define.
+                        // Inside another define, and not under stringification or join.
                         // Can't subst now, or
                         // `define a x,y
                         // foo(`a,`b)  would break because a contains comma
